import warnings
from typing import Generic, TypeVar
from unittest import TestCase

try:
    from pydantic.v1 import BaseModel as PydanticBaseModel, Field
except ImportError:  # pragma: no cover
    from pydantic import BaseModel as PydanticBaseModel, Field  # type: ignore

from fred_types.common import BaseId, BaseModel, GenericModel
from fred_types.exceptions import ExtraArgumentWarning


class BaseModelTest(TestCase):
    def test_warning(self):
        class TestModel(BaseModel):
            name: str

        with warnings.catch_warnings(record=True) as captured:
            warnings.simplefilter("always")

            TestModel(name="Kryten", id="2X4B", ship="Red Dwarf")  # type: ignore[call-arg]

        self.assertEqual(len(captured), 2)
        self.assertIsInstance(captured[0].message, ExtraArgumentWarning)
        self.assertIsInstance(captured[1].message, ExtraArgumentWarning)

    def test_alias(self):
        class TestModel(BaseModel):
            name: str = Field(alias="nickname")

        with warnings.catch_warnings(record=True) as captured:
            warnings.simplefilter("always")

            TestModel(nickname="Kryten")

        self.assertEqual(len(captured), 0)


class GenericModelTest(TestCase):
    def test_warning(self):
        _T = TypeVar("_T", bound=str)

        class TestModel(GenericModel, Generic[_T]):
            name: _T

        with warnings.catch_warnings(record=True) as captured:
            warnings.simplefilter("always")

            TestModel(name="Kryten", id="2X4B", ship="Red Dwarf")  # type: ignore[call-arg]

        self.assertEqual(len(captured), 2)
        self.assertIsInstance(captured[0].message, ExtraArgumentWarning)
        self.assertIsInstance(captured[1].message, ExtraArgumentWarning)


class BaseIdTest(TestCase):
    def test_validation(self):
        class Model(PydanticBaseModel):
            uid: BaseId

        self.assertEqual(Model(uid="007").uid, BaseId("007"))  # type: ignore[arg-type]

    def test_repr(self):
        self.assertEqual(repr(BaseId("007")), "BaseId('007')")
